/*****************************************************************************
 *  diode.c: here are functions for testing diodes
 *  This is part of IC-tester program.
 *  Copyright (C) 2001,2002  Silvio Klaic, sklaic@student.math.hr
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
 *****************************************************************************/

#include <stdio.h>
#include <ctype.h>
#include <string.h>

#include "rw.h"
#include "portlib.h"
#include "device.h"

void diode_test(unsigned int IO)
{
char ch[20];
short int WhaT;

if (allow_io(IO, 2) == -1) {/* test port access */
 fprintf(stderr,"Opa... You aren't being allowed access to the ports 0x%03X\
 and 0x%03X.\n",IO,IO+1);
}else{/* ok access granted continue */
 if(is_power_on(IO)) printf("\n => WARNING: ictester device not powered - can\
 not do check!\n\n");
 else{/* power ok do check */
  enable_port(1,IO);/* enable ictester device */
  printf("Selected port is: 0x%03X.",IO);WhaT=0;/* make find add-on to false */
  if(portwrite(IO,"000============="))/* setting data fail, abort */
   printf("Error! Data aren't successful write to device.\n\
There is problem with transferring data to device, possible low power?\n");
  else{/* compare and analyze data to find add-on */
   portread(IO,ch);ch[16]=0;/* read data */ 
   if(strcmp(ch,"================")==0)/* woha, read error */
    printf("Error! Read data from device are corrupted.\n");
   else{ WhaT=1;/* ok compare data */
    if(strcmp(ch,"0001111111111111")!=0) WhaT=-1;/* continue scan */
    portwrite(IO,"====0=0===0=====");portread(IO,ch);ch[16]=0;/* read data */
    if(strcmp(ch,"0001010111011111")!=0) WhaT=-1;/* continue scan */
    portwrite(IO,"111=0=0===0=====");portread(IO,ch);ch[16]=0;/* read data */
    if(strcmp(ch,"1111000110001111")!=0) WhaT=-1;/* continue scan */
   }/* end else if read ok */
  }/* end else if port write */
  if(WhaT>0){/* all ok, found add on, do test */
   printf(" OCR/Diode add-on detected.\nConnect diode to pins 1 and 3 on\
 OCR/Diode add-on device.\nPress ENTER key to begin test, Q for quit... ");
   if(toupper(getchar())!='Q'){/* ok, do check */
    WhaT=0;portwrite(IO,"0===============");portread(IO,ch);ch[16]=0;
    if(strcmp(ch,"0111111111111111")==0){/* -|<|- or NC */
     portwrite(IO,"==0=============");portread(IO,ch);ch[16]=0;
     if(strcmp(ch,"0101111111111111")==0){ WhaT=3;/* -|<|- do confirm */
      /* confirm with +5V test if diode working correctly */
      portwrite(IO,"1===0=====0=====");portread(IO,ch);ch[16]=0;
      if(strcmp(ch,"1111001111001111")==0){/* ok continue */
       portwrite(IO,"==1=0=====0=====");portread(IO,ch);ch[16]=0;
       if(strcmp(ch,"0111011111001111")!=0) WhaT=9;/* something wrong */
      }else WhaT=5;/* something wrong, not diode connected at pins? */
     }else if(strcmp(ch,"1101111111111111")==0){ WhaT=2;/* LED or NC check */
      /* confirm with +5V test if it's really not connected */
      portwrite(IO,"1===0=====0=====");portread(IO,ch);ch[16]=0;
      if(strcmp(ch,"1101001111011111")==0){/* ok continue */
       portwrite(IO,"==1=0=====0=====");portread(IO,ch);ch[16]=0;
       if(strcmp(ch,"0111011111001111")!=0){ WhaT=5;/* possible LED -|>|- */
	if(strcmp(ch,"1111001111001111")!=0) WhaT=9;/* not LED diode */
       }/* end if possible led */
      }else if(strcmp(ch,"1111001111001111")==0){WhaT=6;/* LED diode -|<|- */
       portwrite(IO,"==1=0=====0=====");portread(IO,ch);ch[16]=0;
       if(strcmp(ch,"0111011111001111")!=0) WhaT=9;/* not LED diode */
      }else WhaT=9;/* something wrong, not diode connected at pins? */
     }else WhaT=9;/* something wrong, not diode connected at pins? */
    }else if(strcmp(ch,"0101111111111111")==0){/* -|>|- or SHORT */
     portwrite(IO,"==0=============");portread(IO,ch);ch[16]=0;
     if(strcmp(ch,"1101111111111111")==0){ WhaT=1;/* -|>|- do confirm */
      /* confirm with +5V test if diode or SHORT working correctly */
      portwrite(IO,"1===0=====0=====");portread(IO,ch);ch[16]=0;
      if(strcmp(ch,"1101001111011111")==0){/* ok continue */
       portwrite(IO,"==1=0=====0=====");portread(IO,ch);ch[16]=0;
       if(strcmp(ch,"1111001111001111")!=0) WhaT=9;/* something wrong */
      }else WhaT=5;/* something wrong, not diode connected at pins? */
     }else if(strcmp(ch,"0101111111111111")==0){WhaT=4;/* SHORT do confirm */
      /* confirm with +5V test if diode or SHORT working correctly */
      portwrite(IO,"1===0=====0=====");portread(IO,ch);ch[16]=0;
      if(strcmp(ch,"1111001111001111")==0){/* ok continue */
       portwrite(IO,"==1=0=====0=====");portread(IO,ch);ch[16]=0;
       if(strcmp(ch,"1111001111001111")!=0) WhaT=9;/* something wrong */
      }else WhaT=9;/* something wrong, not diode connected at pins? */
     }else WhaT=9;/* something wrong, not diode connected at pins? */
    }else WhaT=9;/* something wrong, not diode connected at pins? */
    printf("|  -=-    -=-    -=-  |\n| pin 1  pin 2  pin 3 |\n\
`---|-------------|---'\n");
    switch (WhaT){/* check results */
     case 0:/* error during read or write */
           printf("Error: Detecting power on some pins and there shouldn't be\
 any!\nUse -to option and do test OCR/Diode device to make sure that's\
 working ok.\n");break;
     case 1:/* pin 1 -|>|- pin 3 */
	   printf("    |     |\\ |    |\n    `-----| >|----'\n\
 (Anode)  |/ | (Cathode)\n\nDiode working 100%%.\n\n");break;
     case 2:/* Not Conneccted or burn out */
	   printf("    |\t\t  |\n    `-- \t--'\n\nDiode burn out or not\
 connected to OCR/Diode add-on tester socket.\n\n");break;
     case 3:/* pin 1 -|<|- pin 3 */
	   printf("    |     | /|    |\n    `-----|< |----'\n\
(Cathode) | \\|  (Anode)\n\nDiode working 100%%.\n\n");break;
     case 4:/* short circuit */
	   printf("    |\t\t  |\n    `-------------'\n\nDiode melted,\
 DO NOT USE it anywhere. Pins 1 and 3 are in short circuit.\n\n");break;
     case 5:/* LED pin 1 -|>|- pin 3 */
	   printf("    |     |\\ | // |\n    `-----| >|----'\n\
 (Anode)  |/ | (Cathode)\n\nLED diode working 100%%.\n\n");break;
     case 6:/* LED pin 1 -|<|- pin 3 */
	   printf("    |     | /| // |\n    `-----|< |----'\n\
(Cathode) | \\|  (Anode)\n\nLED diode working 100%%.\n\n");break;
     default:/* error, not diode connected at pins 1 and 3 */
         printf("Error: At test socket isn't diode, aborting test.\n");break;
    }/* end switch */
   }/* end if do check */
  }else if(WhaT==-1)/* add-on device not detected, or not working */
   printf("Error: OCR/Diode add-on not detected or not working properly.\n\
Connect OCR/Diode add-on to test socket on IC-Tester device, or\n\
use -to option and do test add-on device to make sure that's working ok.\n");
  else/* error in reading/sending data to device */
   printf("Error: Detecting power on some pins and there shouldn't be any!\n\
Use -t option and do test IC Tester device to make sure that's working ok.\n");
  enable_port(0,IO);/* reset and disable ictester device */
 }/* end else if powered */
 if (disallow_io(IO, 2) == -1)/* close port */
  fprintf(stderr, "Woha! Can't disengage port.\n\
You could have some problems with this port from now on...\n");
}/* end else if port access */
}/* end diode_test() */

