/*****************************************************************************
 *  triac.c: here are functions for testing triacs
 *  This is part of IC-tester program.
 *  Copyright (C) 2001,2002  Silvio Klaic, sklaic@student.math.hr
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
 *****************************************************************************/

#include <stdio.h>
#include <ctype.h>
#include <string.h>

#include "rw.h"
#include "portlib.h"
#include "device.h"

/* this part isn't finis yet, here is testing if OCR element is triacs, if
 * not then it could be thyristor. Then from here call testing for thyristor */

#ifdef __minix
_PROTOTYPE(short int thyristor_test, (unsigned int IO));
#else
short int thyristor_test(unsigned int IO);
#endif

void triac_test(unsigned int IO)
{
char ch[19];
short int WhaT;

if (allow_io(IO, 2) == -1) {/* test port access */
 fprintf(stderr,"Opa... You aren't being allowed access to the ports 0x%03X\
 and 0x%03X.\n",IO,IO+1);
}else{/* ok access granted continue */
 if(is_power_on(IO)) printf("\n => WARNING: ictester device not powered - can\
 not do check!\n\n");
 else{/* power ok do check */
  enable_port(1,IO);/* enable ictester device */
  printf("Selected port is: 0x%03X.",IO);WhaT=0;/* make find add-on to false */
  if(portwrite(IO,"000============="))/* setting data fail, abort */
   printf("Error! Data aren't successful write to device.\n\
There is problem with transferring data to device, possible low power?\n");
  else{/* compare and analyze data to find add-on */
   portread(IO,ch);ch[16]=0;/* read data */ 
   if(strcmp(ch,"================")==0)/* woha, read error */
    printf("Error! Read data from device are corrupted.\n");
   else{ WhaT=1;/* ok compare data */
    if(strcmp(ch,"0001111111111111")!=0) WhaT=-1;/* continue scan */
    portwrite(IO,"====0=0===0=====");portread(IO,ch);ch[16]=0;/* read data */
    if(strcmp(ch,"0001010111011111")!=0) WhaT=-1;/* continue scan */
    portwrite(IO,"111=0=0===0=====");portread(IO,ch);ch[16]=0;/* read data */
    if(strcmp(ch,"1111000110001111")!=0) WhaT=-1;/* continue scan */
   }/* end else if read ok */
  }/* end else if port write */
  if(WhaT>0){/* all ok, found add on, do test */
   printf(" OCR/Diode add-on detected.\nConnect OCR element to pins 1, 2 and 3\
 on OCR/Diode add-on device.\nPress ENTER key to begin test, Q for quit... ");
   if(toupper(getchar())!='Q'){portwrite(IO,"================");/* do check */
    WhaT=0;portwrite(IO,"0===============");portread(IO,ch);ch[16]=0;
    if(strcmp(ch,"0101111111111111")==0){/* A1-A2-G or G-A2-A1 */
     WhaT=1;portwrite(IO,"=0==============");portread(IO,ch);ch[16]=0;
     if(strcmp(ch,"1011111111111111")==0){/* continue scaning */
      portwrite(IO,"==0=============");portread(IO,ch);ch[16]=0;
      if(strcmp(ch,"0101111111111111")==0){/* continue scaning */
       /* check if under +5V working as A1-A2-G or G-A2-A1 */
       WhaT=10;portwrite(IO,"================");/* reset input power */
       portwrite(IO,"=10=0=0===0=====");portread(IO,ch);ch[16]=0;
       ch[0]='=';ch[1]='=';ch[2]='=';/* ignore set bits */
       if(strcmp(ch,"===1010110011111")==0){/* + signal set */
        portwrite(IO,"110=0=0===0=====");portread(IO,ch);ch[16]=0;
        ch[0]='=';ch[1]='=';ch[2]='=';/* ignore set bits */
        if(strcmp(ch,"===1000110001111")==0){/* + signal to G */
         portwrite(IO,"=10=0=0===0=====");portread(IO,ch);ch[16]=0;
         ch[0]='=';ch[1]='=';ch[2]='=';/* ignore set bits */
         if(strcmp(ch,"===1000110001111")==0) WhaT=12;/* + signal hold */
	 else if(strcmp(ch,"===1010110011111")==0) WhaT=1;/* short */
	 else WhaT=11;/* + hold not working */
        }else{ WhaT=thyristor_test(IO);/* check if this is thyristor */
        }/* end if + signal to G */
       }/* end if + signal set */
       /* ok + direction working, check - direction */
       portwrite(IO,"================");/* reset input power */
       portwrite(IO,"=01=0=0===0=====");portread(IO,ch);ch[16]=0;
       ch[0]='=';ch[1]='=';ch[2]='=';/* ignore set bits */
       if(strcmp(ch,"===1000111001111")==0){/* - signal set */
        portwrite(IO,"001=0=0===0=====");portread(IO,ch);ch[16]=0;
        ch[0]='=';ch[1]='=';ch[2]='=';/* ignore set bits */
        if(strcmp(ch,"===1010111001111")==0){/* - signal to G */
         portwrite(IO,"=01=0=0===0=====");portread(IO,ch);ch[16]=0;
         ch[0]='=';ch[1]='=';ch[2]='=';/* ignore set bits */
         if(strcmp(ch,"===1000110001111")==0) WhaT+=3;/* - signal hold */
	 else if((WhaT<99)&&(strcmp(ch,"===1000111001111")==0))
	  WhaT=thyristor_test(IO); /* possible thyristor do check */
	 /* check if this is thyristor */
	 else if(WhaT!=99) WhaT++;/* - hold not working */
	}else if((WhaT<99)&&(strcmp(ch,"===1000111001111")==0)){ 
	 WhaT+=3;/* short */
        }/* end if - signal to G */
       }/* end if - signal set */
      }/* end if continue */
     }/* end if continue */
    }else if(strcmp(ch,"0011111111111111")==0){/* Triac A2-A1-G or A2-G-A1 */
     WhaT=1;portwrite(IO,"=0==============");portread(IO,ch);ch[16]=0;
     if(strcmp(ch,"0011111111111111")==0){/* continue scaning */
      portwrite(IO,"==0=============");portread(IO,ch);ch[16]=0;
      if(strcmp(ch,"1101111111111111")==0){/* continue scaning */
       /* check if under +5V working as A2-A1-G or A2-G-A1 */
       WhaT=20;portwrite(IO,"================");/* reset input power */
       portwrite(IO,"=01=0=0===0=====");portread(IO,ch);ch[16]=0;
       ch[0]='=';ch[1]='=';ch[2]='=';/* ignore set bits */
       if(strcmp(ch,"===1010111001111")==0){/* + signal set */
        portwrite(IO,"101=0=0===0=====");portread(IO,ch);ch[16]=0;
        ch[0]='=';ch[1]='=';ch[2]='=';/* ignore set bits */
        if(strcmp(ch,"===1000110001111")==0){/* + signal to G */
         portwrite(IO,"=01=0=0===0=====");portread(IO,ch);ch[16]=0;
         ch[0]='=';ch[1]='=';ch[2]='=';/* ignore set bits */
         if(strcmp(ch,"===1000110001111")==0) WhaT=22;/* + signal hold */
	 else if(strcmp(ch,"===1010111001111")==0) WhaT=1;/* short */
	 else WhaT=21;/* + hold not working */
        }else{ WhaT=thyristor_test(IO);/* check if this is thyristor */
	}/* end else if + signal to G */
       }/* end if + signal set */
       /* ok + direction working, check - direction */
       portwrite(IO,"================");/* reset input power */
       portwrite(IO,"=10=0=0===0=====");portread(IO,ch);ch[16]=0;
       ch[0]='=';ch[1]='=';ch[2]='=';/* ignore set bits */
       if(strcmp(ch,"===1000110011111")==0){/* - signal set */
        portwrite(IO,"010=0=0===0=====");portread(IO,ch);ch[16]=0;
        ch[0]='=';ch[1]='=';ch[2]='=';/* ignore set bits */
        if(strcmp(ch,"===1010110011111")==0){/* - signal to G */
         portwrite(IO,"=10=0=0===0=====");portread(IO,ch);ch[16]=0;
         ch[0]='=';ch[1]='=';ch[2]='=';/* ignore set bits */
         if(strcmp(ch,"===1000110001111")==0) WhaT+=3;/* - signal hold */
	 else if((WhaT<99)&&(strcmp(ch,"===1000110011111")==0))
	  WhaT=thyristor_test(IO); /* possible thyristor do check */
	 else if(WhaT!=99) WhaT++;/* - hold not working */
	}else if((WhaT<99)&&(strcmp(ch,"===1000110011111")==0)){ 
	 WhaT+=2;/* short */
        }/* end if - signal to G */
       }/* end if - signal set */
      }/* end if continue */
     }/* end if continue */
    }else if(strcmp(ch,"0111111111111111")==0){/* Triac A1-G-A2 or G-A1-A2 */
     WhaT=1;portwrite(IO,"=0==============");portread(IO,ch);ch[16]=0;
     if(strcmp(ch,"1001111111111111")==0){/* continue scaning */
      portwrite(IO,"==0=============");portread(IO,ch);ch[16]=0;
      if(strcmp(ch,"1001111111111111")==0){/* continue scaning */
       /* check if under +5V working as A1-G-A2 or G-A1-A2 */
       WhaT=30;portwrite(IO,"================");/* reset input power */
       portwrite(IO,"1=0=0=0===0=====");portread(IO,ch);ch[16]=0;
       ch[0]='=';ch[1]='=';ch[2]='=';/* ignore set bits */
       if(strcmp(ch,"===1000111011111")==0){/* + signal set */
        portwrite(IO,"110=0=0===0=====");portread(IO,ch);ch[16]=0;
        ch[0]='=';ch[1]='=';ch[2]='=';/* ignore set bits */
        if(strcmp(ch,"===1000110001111")==0){/* + signal to G */
         portwrite(IO,"1=0=0=0===0=====");portread(IO,ch);ch[16]=0;
         ch[0]='=';ch[1]='=';ch[2]='=';/* ignore set bits */
         if(strcmp(ch,"===1000110001111")==0) WhaT=32;/* + signal hold */
	 else if(strcmp(ch,"===1000111011111")==0) WhaT=1;/* short */
	 else WhaT=31;/* + hold not working */
        }else{ WhaT=thyristor_test(IO);/* check if this is thyristor */
        }/* end if + signal to G */
       }/* end if + signal set */
       /* ok + direction working, check - direction */
       portwrite(IO,"================");/* reset input power */
       portwrite(IO,"0=1=0=0===0=====");portread(IO,ch);ch[16]=0;
       ch[0]='=';ch[1]='=';ch[2]='=';/* ignore set bits */
       if(strcmp(ch,"===1010110001111")==0){/* - signal set */
        portwrite(IO,"001=0=0===0=====");portread(IO,ch);ch[16]=0;
        ch[0]='=';ch[1]='=';ch[2]='=';/* ignore set bits */
        if(strcmp(ch,"===1010111001111")==0){/* - signal to G */
         portwrite(IO,"0=1=0=0===0=====");portread(IO,ch);ch[16]=0;
         ch[0]='=';ch[1]='=';ch[2]='=';/* ignore set bits */
         if(strcmp(ch,"===1000110001111")==0) WhaT+=3;/* - signal hold */
	 else if((WhaT<99)&&(strcmp(ch,"===1010110001111")==0))
	  WhaT=thyristor_test(IO); /* possible thyristor do check */
	 else if(WhaT!=99) WhaT++;/* - hold not working */
	}else if((WhaT<99)&&(strcmp(ch,"===1010110001111")==0)){ 
	 WhaT++;/* short */
        }/* end if - signal to G */
       }/* end if - signal set */
      }/* end if continue */
     }else if(strcmp(ch,"1011111111111111")==0){/* possible burn out or NC? */
      portwrite(IO,"==0=============");portread(IO,ch);ch[16]=0;
      if(strcmp(ch,"1101111111111111")==0) WhaT=1;/* NC or burn out */
     }/* end if continue */
    }else if(strcmp(ch,"0001111111111111")==0) WhaT=5;/* short, end if triac */
    if(WhaT<99) printf("|  -=-    -=-    -=-  |\n| pin 1  pin 2  pin 3 |\n\
`---|------|------|---'\n");
    if((WhaT>=10)&&(WhaT<20)) printf("   (?)   (A2)    (?)\n    |     -+--    |\
\n    |     \\//\\    |\n    |     -+--    |\n    `------'  `---'\n\n");
    if((WhaT>=20)&&(WhaT<30)) printf("  (A2)    (?)    (?)\n    |     -+--    |\
\n    |     \\//\\    |\n    |     -+--    |\n    `------'  `---'\n\n");
    if((WhaT>=30)&&(WhaT<40)) printf("   (?)    (?)   (A2)\n    |      |      |\
\n    |      `-|>|  |\n    `--------|<|--'\n\n");
    switch (WhaT){/* check rezults */
     case 0:/* error during read or write */
           printf("Error: Detecting power on some pins and there shouldn't be\
 any!\nUse -to option and do test OCR/Diode device to make sure that's\
 working ok.\n");break;
     case 1:/* triac: ??? Not working */
	    printf("\nOCR element not connected properly on test socket or\
 it's burn out.\n\n");break;
     case 2:/* pin 1 and 2 in short */
	    printf("    `------'\n\nTest pins 1 and 2 are in short circuit,\
 it's possible that OCR element is melted\nor not connected properly at test\
 socket!\n\n");break;
     case 3:/* pin 2 and 3 in short */
	    printf("\t   `------'\n\nTest pins 2 and 3 are in short circuit,\
 it's possible that OCR element is melted\nor not connected properly at test\
 socket!\n\n");break;
     case 4:/* pin 1 and 3 in short */
	    printf("    `-------------'\n\nTest pins 1 and 3 are in short\
 circuit, it's possible that OCR element is melted\nor not connected properly\
 at test socket!\n\n");break;
     case 5:/* pin 1,2 and 3 in short */
	    printf("    `------^------'\n\nAll test pins are in short circuit,\
 OCR element is melted!\nDo not use this element anywhere!\n\n");break;
     case 10:/* triac: -A2- not working */
	    printf("Triac not working.\n");break;
     case 11:/* triac: -A2- + hold not working - hold fail */
	    printf("Triac not working. (+) A2 to A1 not working with DC\
 current.\nHave not response for (-) A1 to A2, do NOT use this triac.\n");break;
     case 12:/* triac: -A2- - hold fail */
	    printf("Triac not working. Have not response for (-) A1 to A2,\
 do NOT use this triac.\n");break;
     case 13:/* triac: -A2- - hold not working */
	    printf("Triac not working. (-) A1 to A2 not working with DC\
 current.\nDo NOT use this triac.\n");break;
     case 14:/* triac: -A2- + hold not working */
	    printf("Triac not working. Have not response for (+) A2 to A1\
\nDo NOT use this triac.\n");break;
     case 15:/* triac: -A2- working ok */
	   printf("Triac working 100%% :-)\n\n");break;
     case 20:/* triac: A2-- not working */
	    printf("Triac not working.\n");break;
     case 21:/* triac: A2-- + hold not working - hold fail */
	    printf("Triac not working. (+) A2 to A1 not working with DC\
 current.\nHave not response for (-) A1 to A2, do NOT use this triac.\n");break;
     case 22:/* triac: A2-- - hold fail */
	    printf("Triac not working. Have not response for (-) A1 to A2,\
 do NOT use this triac.\n");break;
     case 23:/* triac: A2-- - hold not working */
	    printf("Triac not working. (-) A1 to A2 not working with DC\
 current.\nDo NOT use this triac.\n");break;
     case 24:/* triac: A2-- + hold not working */
	    printf("Triac not working. Have not response for (+) A2 to A1\
\nDo NOT use this triac.\n");break;
     case 25:/* triac: A2-- working ok */
	   printf("Triac working 100%% :-)\n\n");break;
     case 30:/* triac: --A2 not working */
	    printf("Triac not working.\n");break;
     case 31:/* triac: --A2 + hold not working - hold fail */
	    printf("Triac not working. (+) A2 to A1 not working with DC\
 current.\nHave not response for (-) A1 to A2, do NOT use this triac.\n");break;
     case 32:/* triac: --A2 - hold fail */
	    printf("Triac not working. Have not response for (-) A1 to A2,\
 do NOT use this triac.\n");break;
     case 33:/* triac: --A2 - hold not working */
	    printf("Triac not working. (-) A1 to A2 not working with DC\
 current.\nDo NOT use this triac.\n");break;
     case 34:/* triac: --A2 + hold not working */
	    printf("Triac not working. Have not response for (+) A2 to A1\
\nDo NOT use this triac.\n");break;
     case 35:/* triac: --A2 working ok */
	   printf("Triac working 100%% :-)\n\n");break;
     case 99:break;/* skip this one - possible thyristor */
     default:/* error, not OCR connected at pins 1, 2 and 3 */
            printf("Error (%d): At test socket isn't OCR element, aborting test.\n",WhaT);
	    break;
    }/* end switch */
   }/* end if do check */
  }else if(WhaT==-1)/* add-on device not detected, or not working */
   printf("Error: OCR/Diode add-on not detected or not working properly.\n\
Connect OCR/Diode add-on to test socket on IC-Tester device, or\n\
use -to option and do test add-on device to make sure that's working ok.\n");
  else/* error in reading/sending data to device */
   printf("Error: Detecting power on some pins and there shouldn't be any!\n\
Use -t option and do test IC Tester device to make sure that's working ok.\n");
  enable_port(0,IO);/* reset and disable ictester device */
 }/* end else if powered */
 if (disallow_io(IO, 2) == -1)/* close port */
  fprintf(stderr, "Woha! Can't disengage port.\n\
You culd have some problems with this port from now on...\n");
}/* end else if port access */
}/* end triac_test() */

